## upterm host

Host a terminal session

### Synopsis

Host a terminal session over a reverse SSH tunnel to the Upterm server with the IO of the host and the client attaching to the IO of a command. By default, the command authenticates against the Upterm server using the private key files located at ~/.ssh/id_dsa, ~/.ssh/id_ecdsa, ~/.ssh/id_ed25519, and ~/.ssh/id_rsa. If no private key file is found, the command reads private keys from SSH Agent. If no private key is found from files or SSH Agent, the command falls back to generate one on the fly. To permit authorized clients to join, client public keys can be specified with --authorized-key.

```
upterm host [flags]
```

### Examples

```
  # Host a terminal session that runs $SHELL with
  # client's input/output attaching to the host's
  upterm host

  # Host a terminal session that only allows specified public key(s) to connect
  upterm host --authorized-key PATH_TO_PUBLIC_KEY

  # Host a session with a custom command.
  upterm host -- docker run --rm -ti ubuntu bash

  # Host a session that runs 'tmux new -t pair-programming' and
  # force clients to join with 'tmux attach -t pair-programming'.
  # This is similar to tmate.
  upterm host --force-command 'tmux attach -t pair-programming' -- tmux new -t pair-programming

  # Use a different Uptermd server and host a session via WebSocket
  upterm host --server wss://YOUR_UPTERMD_SERVER -- YOUR_COMMAND
```

### Options

```
  -a, --authorized-key string   an authorized_keys file that lists public keys that are permitted to connect.
  -f, --force-command string    force execution of a command and attach its input/output to client's.
      --github-user strings     this GitHub user public keys are permitted to connect.
      --gitlab-user strings     this GitLab user public keys are permitted to connect.
  -h, --help                    help for host
      --known-hosts string      a file contains the known keys for remote hosts (required). (default "/Users/owen/.ssh/known_hosts")
  -i, --private-key strings     private key file for public key authentication against the upterm server (default [/Users/owen/.ssh/id_ed25519])
  -r, --read-only               host a read-only session. Clients won't be able to interact.
      --server string           upterm server address (required), supported protocols are ssh, ws, or wss. (default "ssh://uptermd.upterm.dev:22")
```

### SEE ALSO

* [upterm](upterm.md)	 - Secure Terminal Sharing

###### Auto generated by spf13/cobra on 3-Dec-2022
